@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf8 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -19,12 +19,11 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-from builtins import range
 import os
-import duplicity.backend
-from duplicity import util
 
-hsi_command = u"hsi"
+import duplicity.backend
+
+hsi_command = "hsi"
 
 
 class HSIBackend(duplicity.backend.Backend):
@@ -33,37 +32,37 @@ class HSIBackend(duplicity.backend.Backend):
         self.host_string = parsed_url.hostname
         self.remote_dir = parsed_url.path
         if self.remote_dir:
-            self.remote_prefix = self.remote_dir + u"/"
+            self.remote_prefix = f"{self.remote_dir}/"
         else:
-            self.remote_prefix = u""
+            self.remote_prefix = ""
 
     def _put(self, source_path, remote_filename):
-        if isinstance(remote_filename, b"".__class__):
-            remote_filename = util.fsdecode(remote_filename)
-        commandline = u'%s "put %s : %s%s"' % (hsi_command, source_path.uc_name, self.remote_prefix, remote_filename)
+        if isinstance(remote_filename, bytes):
+            remote_filename = os.fsdecode(remote_filename)
+        commandline = f'{hsi_command} "put {source_path.uc_name} : {self.remote_prefix}{remote_filename}"'
         self.subprocess_popen(commandline)
 
     def _get(self, remote_filename, local_path):
-        if isinstance(remote_filename, b"".__class__):
-            remote_filename = util.fsdecode(remote_filename)
-        commandline = u'%s "get %s : %s%s"' % (hsi_command, local_path.uc_name, self.remote_prefix, remote_filename)
+        if isinstance(remote_filename, bytes):
+            remote_filename = os.fsdecode(remote_filename)
+        commandline = f'{hsi_command} "get {local_path.uc_name} : {self.remote_prefix}{remote_filename}"'
         self.subprocess_popen(commandline)
 
     def _list(self):
-        commandline = u'%s "ls -l %s"' % (hsi_command, self.remote_dir)
+        commandline = f'{hsi_command} "ls -l {self.remote_dir}"'
         l = self.subprocess_popen(commandline)[2]
         l = l.split(os.linesep.encode())[3:]
         for i in range(0, len(l)):
             if l[i]:
                 l[i] = l[i].split()[-1]
-        return [util.fsencode(x) for x in l if x]
+        return [os.fsencode(x) for x in l if x]
 
     def _delete(self, filename):
-        if isinstance(filename, b"".__class__):
-            filename = util.fsdecode(filename)
-        commandline = u'%s "rm %s%s"' % (hsi_command, self.remote_prefix, filename)
+        if isinstance(filename, bytes):
+            filename = os.fsdecode(filename)
+        commandline = f'{hsi_command} "rm {self.remote_prefix}{filename}"'
         self.subprocess_popen(commandline)
 
 
-duplicity.backend.register_backend(u"hsi", HSIBackend)
-duplicity.backend.uses_netloc.extend([u'hsi'])
+duplicity.backend.register_backend("hsi", HSIBackend)
+duplicity.backend.uses_netloc.extend(["hsi"])
