@@ -20,19 +20,20 @@
 
 import os
 import re
+
 from duplicity import backend
-from duplicity.errors import BackendException
-from duplicity import log
 from duplicity import config
-from duplicity import util
+from duplicity import log
+from duplicity.errors import BackendException
 
 
 class Par2Backend(backend.Backend):
-    u"""This backend wrap around other backends and create Par2 recovery files
+    """This backend wrap around other backends and create Par2 recovery files
     before the file and the Par2 files are transfered with the wrapped backend.
 
     If a received file is corrupt it will try to repair it on the fly.
     """
+
     def __init__(self, parsed_url):
         backend.Backend.__init__(self, parsed_url)
 
@@ -48,15 +49,25 @@ class Par2Backend(backend.Backend):
             self.volumes = 1
 
         try:
-            self.common_options = config.par2_options + u" -q -q"
+            self.common_options = f"{config.par2_options} -q -q"
         except AttributeError:
-            self.common_options = u"-q -q"
+            self.common_options = "-q -q"
 
         self.wrapped_backend = backend.get_backend_object(parsed_url.url_string)
 
-        for attr in [u'_get', u'_put', u'_list', u'_delete', u'_delete_list',
-                     u'_query', u'_query_list', u'_retry_cleanup', u'_error_code',
-                     u'_move', u'_close']:
+        for attr in [
+            "_get",
+            "_put",
+            "_list",
+            "_delete",
+            "_delete_list",
+            "_query",
+            "_query_list",
+            "_retry_cleanup",
+            "_error_code",
+            "_move",
+            "_close",
+        ]:
             if hasattr(self.wrapped_backend, attr):
                 setattr(self, attr, getattr(self, attr[1:]))
 
@@ -65,7 +76,7 @@ class Par2Backend(backend.Backend):
         self._delete_list = self.delete_list
 
     def transfer(self, method, source_path, remote_filename):
-        u"""create Par2 files and transfer the given file and the Par2 files
+        """create Par2 files and transfer the given file and the Par2 files
         with the wrapped backend.
 
         Par2 must run on the real filename or it would restore the
@@ -77,23 +88,26 @@ class Par2Backend(backend.Backend):
         source_symlink = par2temp.append(remote_filename)
         source_target = source_path.get_canonical()
         if not os.path.isabs(source_target):
-            source_target = os.path.join(util.fsencode(os.getcwd()), source_target)
+            source_target = os.path.join(os.fsencode(os.getcwd()), source_target)
         os.symlink(source_target, source_symlink.get_canonical())
         source_symlink.setdata()
 
-        log.Info(u"Create Par2 recovery files")
-        par2create = u'par2 c -r%d -n%d %s "%s"' % (self.redundancy, self.volumes,
-                                                    self.common_options,
-                                                    util.fsdecode(source_symlink.get_canonical()))
+        log.Info("Create Par2 recovery files")
+        par2create = (
+            f"par2 c -r{int(self.redundancy)} -n{int(self.volumes)} {self.common_options} "
+            f'"{os.fsdecode(source_symlink.get_canonical())}"'
+        )
         returncode, out, err = self.subprocess_popen(par2create)
 
         if returncode:
-            log.Warn(u"Failed to create par2 file with requested options, retrying with -n1")
-            par2create = u'par2 c -r%d -n1 %s "%s"' % (self.redundancy, self.common_options,
-                                                       util.fsdecode(source_symlink.get_canonical()))
+            log.Warn("Failed to create par2 file with requested options, retrying with -n1")
+            par2create = (
+                f"par2 c -r{int(self.redundancy)} -n1 {self.common_options} "
+                f'"{os.fsdecode(source_symlink.get_canonical())}"'
+            )
             returncode, out, err = self.subprocess_popen(par2create)
             if not returncode:
-                log.Warn(u"Successfully created par2 file with -n1")
+                log.Warn("Successfully created par2 file with -n1")
 
         source_symlink.delete()
         files_to_transfer = []
@@ -101,7 +115,7 @@ class Par2Backend(backend.Backend):
             for file in par2temp.listdir():
                 files_to_transfer.append(par2temp.append(file))
         else:
-            log.Error(u"FAILED to create par2 file with returncode %d" % returncode)
+            log.Error(f"FAILED to create par2 file with returncode {int(returncode)}")
 
         method(source_path, remote_filename)
         for file in files_to_transfer:
@@ -116,7 +130,7 @@ class Par2Backend(backend.Backend):
         self.transfer(self.wrapped_backend._move, local, remote)
 
     def get(self, remote_filename, local_path):
-        u"""transfer remote_filename and the related .par2 file into
+        """transfer remote_filename and the related .par2 file into
         a temp-dir. remote_filename will be renamed into local_path before
         finishing.
 
@@ -131,32 +145,34 @@ class Par2Backend(backend.Backend):
         self.wrapped_backend._get(remote_filename, local_path_temp)
 
         try:
-            par2file = par2temp.append(remote_filename + b'.par2')
+            par2file = par2temp.append(remote_filename + b".par2")
             self.wrapped_backend._get(par2file.get_filename(), par2file)
 
-            par2verify = u'par2 v %s %s "%s"' % (self.common_options,
-                                                 util.fsdecode(par2file.get_canonical()),
-                                                 util.fsdecode(local_path_temp.get_canonical()))
+            par2verify = (
+                f"par2 v {self.common_options} {os.fsdecode(par2file.get_canonical())} "
+                f'"{os.fsdecode(local_path_temp.get_canonical())}"'
+            )
             returncode, out, err = self.subprocess_popen(par2verify)
 
             if returncode:
-                log.Warn(u"File is corrupt. Try to repair %s" % remote_filename)
-                c = re.compile(u'%s\\.vol[\\d+]*\\.par2' % remote_filename.decode())
-                par2volumes = [f for f in self.wrapped_backend._list() if c.match(util.fsdecode(f))]
+                log.Warn(f"File is corrupt. Try to repair {remote_filename}")
+                c = re.compile(f"{remote_filename.decode()}\\.vol[\\d+]*\\.par2")
+                par2volumes = [f for f in self.wrapped_backend._list() if c.match(os.fsdecode(f))]
 
                 for filename in par2volumes:
                     file = par2temp.append(filename)
                     self.wrapped_backend._get(filename, file)
 
-                par2repair = u'par2 r %s %s "%s"' % (self.common_options,
-                                                     util.fsdecode(par2file.get_canonical()),
-                                                     util.fsdecode(local_path_temp.get_canonical()))
+                par2repair = (
+                    f"par2 r {self.common_options} {os.fsdecode(par2file.get_canonical())} "
+                    f'"{os.fsdecode(local_path_temp.get_canonical())}"'
+                )
                 returncode, out, err = self.subprocess_popen(par2repair)
 
                 if returncode:
-                    log.Error(u"Failed to repair %s" % remote_filename)
+                    log.Error(f"Failed to repair {remote_filename}")
                 else:
-                    log.Warn(u"Repair successful %s" % remote_filename)
+                    log.Warn(f"Repair successful {remote_filename}")
         except BackendException:
             # par2 file not available
             pass
@@ -165,47 +181,45 @@ class Par2Backend(backend.Backend):
             par2temp.deltree()
 
     def delete(self, filename):
-        u"""delete given filename and its .par2 files
-        """
+        """delete given filename and its .par2 files"""
         self.wrapped_backend._delete(filename)
 
         remote_list = self.unfiltered_list()
 
-        c = re.compile(u'%s(?:\\.vol[\\d+]*)?\\.par2' % util.fsdecode(filename))
+        c = re.compile(f"{os.fsdecode(filename)}(?:\\.vol[\\d+]*)?\\.par2")
         for remote_filename in remote_list:
-            if c.match(util.fsdecode(remote_filename)):
-                self.wrapped_backend._delete(util.fsencode(remote_filename))
+            if c.match(os.fsdecode(remote_filename)):
+                self.wrapped_backend._delete(os.fsencode(remote_filename))
 
     def delete_list(self, filename_list):
-        u"""delete given filename_list and all .par2 files that belong to them
-        """
+        """delete given filename_list and all .par2 files that belong to them"""
         remote_list = self.unfiltered_list()
 
         for filename in filename_list[:]:
-            c = re.compile(u'%s(?:\\.vol[\\d+]*)?\\.par2' % util.fsdecode(filename))
+            c = re.compile(f"{os.fsdecode(filename)}(?:\\.vol[\\d+]*)?\\.par2")
             for remote_filename in remote_list:
-                if c.match(util.fsdecode(remote_filename)):
+                if c.match(os.fsdecode(remote_filename)):
                     # insert here to make sure par2 files will be removed first
                     filename_list.insert(0, remote_filename)
 
-        if hasattr(self.wrapped_backend, u'_delete_list'):
+        if hasattr(self.wrapped_backend, "_delete_list"):
             return self.wrapped_backend._delete_list(filename_list)
         else:
             for filename in filename_list:
                 self.wrapped_backend._delete(filename)
 
     def list(self):
-        u"""
+        """
         Return list of filenames (byte strings) present in backend
 
         Files ending with ".par2" will be excluded from the list.
         """
         remote_list = self.wrapped_backend._list()
 
-        c = re.compile(u'(?!.*\\.par2$)')
+        c = re.compile("(?!.*\\.par2$)")
         filtered_list = []
         for filename in remote_list:
-            if c.match(util.fsdecode(filename)):
+            if c.match(os.fsdecode(filename)):
                 filtered_list.append(filename)
         return filtered_list
 
@@ -228,4 +242,4 @@ class Par2Backend(backend.Backend):
         self.wrapped_backend._close()
 
 
-backend.register_backend_prefix(u'par2', Par2Backend)
+backend.register_backend_prefix("par2", Par2Backend)
