@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -21,99 +21,94 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
+
 import base64
-import httplib
+import http.client
 import os
 import re
-import urllib
-import urllib2
-import urlparse
+import shutil
+import urllib.error
+import urllib.parse
+import urllib.request
 import xml.dom.minidom
 
 import duplicity.backend
-from duplicity import globals
+from duplicity import config
 from duplicity import log
 from duplicity import util
-from duplicity.errors import BackendException, FatalBackendException
+from duplicity.errors import (
+    BackendException,
+    FatalBackendException,
+)
 
 
-class CustomMethodRequest(urllib2.Request):
+class CustomMethodRequest(urllib.request.Request):
     """
     This request subclass allows explicit specification of
-    the HTTP request method. Basic urllib2.Request class
+    the HTTP request method. Basic urllib.request.Request class
     chooses GET or POST depending on self.has_data()
     """
+
     def __init__(self, method, *args, **kwargs):
         self.method = method
-        urllib2.Request.__init__(self, *args, **kwargs)
+        urllib.request.Request.__init__(self, *args, **kwargs)
 
     def get_method(self):
         return self.method
 
 
-class VerifiedHTTPSConnection(httplib.HTTPSConnection):
-        def __init__(self, *args, **kwargs):
-            try:
-                global socket, ssl
-                import socket
-                import ssl
-            except ImportError:
-                raise FatalBackendException(_("Missing socket or ssl python modules."))
+class CustomHTTPSConnection(http.client.HTTPSConnection):
+    def __init__(self, *args, **kwargs):
+        try:
+            global socket, ssl
+            import socket
+            import ssl
+        except ImportError:
+            raise FatalBackendException(_("Missing socket or ssl python modules."))
 
-            httplib.HTTPSConnection.__init__(self, *args, **kwargs)
+        http.client.HTTPSConnection.__init__(self, *args, **kwargs)
 
-            self.cacert_file = globals.ssl_cacert_file
-            self.cacert_candidates = ["~/.duplicity/cacert.pem",
-                                      "~/duplicity_cacert.pem",
-                                      "/etc/duplicity/cacert.pem"]
-            # if no cacert file was given search default locations
-            if not self.cacert_file:
-                for path in self.cacert_candidates:
-                    path = os.path.expanduser(path)
-                    if (os.path.isfile(path)):
-                        self.cacert_file = path
-                        break
+        self.cacert_file = config.ssl_cacert_file
+        self.cacert_candidates = [
+            "~/.duplicity/cacert.pem",
+            "~/duplicity_cacert.pem",
+            "/etc/duplicity/cacert.pem",
+        ]
+        # if no cacert file was given search default locations
+        if not self.cacert_file:
+            for path in self.cacert_candidates:
+                path = os.path.expanduser(path)
+                if os.path.isfile(path):
+                    self.cacert_file = path
+                    break
 
-            # check if file is accessible (libssl errors are not very detailed)
-            if self.cacert_file and not os.access(self.cacert_file, os.R_OK):
-                raise FatalBackendException(_("Cacert database file '%s' is not readable.") % self.cacert_file)
+        # check if file is accessible (libssl errors are not very detailed)
+        if self.cacert_file and not os.access(self.cacert_file, os.R_OK):
+            raise FatalBackendException(_("Cacert database file '%s' is not readable.") % self.cacert_file)
 
-        def connect(self):
-            # create new socket
-            sock = socket.create_connection((self.host, self.port),
-                                            self.timeout)
-            if self._tunnel_host:
-                self.sock = sock
-                self.tunnel()
+    def connect(self):
+        # create new socket
+        sock = socket.create_connection((self.host, self.port), self.timeout)
+        if self._tunnel_host:
+            self.sock = sock
+            self.tunnel()
 
-            # python 2.7.9+ supports default system certs now
-            if "create_default_context" in dir(ssl):
-                context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH, cafile=self.cacert_file, capath=globals.ssl_cacert_path)
-                self.sock = context.wrap_socket(sock, server_hostname=self.host)
-            # the legacy way needing a cert file
-            else:
-                if globals.ssl_cacert_path:
-                    raise FatalBackendException(_("Option '--ssl-cacert-path' is not supported with python 2.7.8 and below."))
+        if config.ssl_no_check_certificate:
+            context = ssl._create_unverified_context()
+        else:
+            context = ssl.create_default_context(
+                ssl.Purpose.SERVER_AUTH,
+                cafile=self.cacert_file,
+                capath=config.ssl_cacert_path,
+            )
+        self.sock = context.wrap_socket(sock, server_hostname=self.host)
 
-                if not self.cacert_file:
-                    raise FatalBackendException(_("""\
-For certificate verification with python 2.7.8 or earlier a cacert database file is needed in one of these locations: %s
-Hints:
-  Consult the man page, chapter 'SSL Certificate Verification'.
-  Consider using the options --ssl-cacert-file, --ssl-no-check-certificate .""") % ", ".join(self.cacert_candidates))
-
-                # wrap the socket in ssl using verification
-                self.sock = ssl.wrap_socket(sock,
-                                            cert_reqs=ssl.CERT_REQUIRED,
-                                            ca_certs=self.cacert_file,
-                                            )
-
-        def request(self, *args, **kwargs):  # pylint: disable=method-hidden
-            try:
-                return httplib.HTTPSConnection.request(self, *args, **kwargs)
-            except ssl.SSLError as e:
-                # encapsulate ssl errors
-                raise BackendException("SSL failed: %s" % util.uexc(e), log.ErrorCode.backend_error)
+    def request(self, *args, **kwargs):  # pylint: disable=method-hidden
+        try:
+            return http.client.HTTPSConnection.request(self, *args, **kwargs)
+        except ssl.SSLError as e:
+            # encapsulate ssl errors
+            raise BackendException(f"SSL failed: {util.uexc(e)}", log.ErrorCode.backend_error)
 
 
 class WebDAVBackend(duplicity.backend.Backend):
@@ -128,9 +123,24 @@ class WebDAVBackend(duplicity.backend.Backend):
     listbody = '<?xml version="1.0"?><D:propfind xmlns:D="DAV:"><D:prop><D:resourcetype/></D:prop></D:propfind>'
 
     """Connect to remote store using WebDAV Protocol"""
+
     def __init__(self, parsed_url):
         duplicity.backend.Backend.__init__(self, parsed_url)
-        self.headers = {'Connection': 'keep-alive'}
+
+        self.headers = {
+            "Connection": "keep-alive",
+            "Content-Type": "application/octet-stream",
+        }
+        if config.webdav_headers:
+            try:
+                self.headers = util.merge_dicts(self.headers, util.csv_args_to_dict(config.webdav_headers))
+            except IndexError as e:
+                log.FatalError("--webdav-headers value has an odd number of arguments.  Must be paired.")
+            except SyntaxError as e:
+                log.FatalError("--webdav-headers value has bad syntax.  Check quoting pairs.")
+            except Exception as e:
+                log.FatalErrof(f"--webdav-headers value caused error: {e}")
+
         self.parsed_url = parsed_url
         self.digest_challenge = None
         self.digest_auth_handler = None
@@ -139,7 +149,6 @@ class WebDAVBackend(duplicity.backend.Backend):
         self.password = self.get_password()
         self.directory = self.sanitize_path(parsed_url.path)
 
-        log.Info(_("Using WebDAV protocol %s") % (globals.webdav_proto,))
         log.Info(_("Using WebDAV host %s port %s") % (parsed_url.hostname, parsed_url.port))
         log.Info(_("Using WebDAV directory %s") % (self.directory,))
 
@@ -147,10 +156,10 @@ class WebDAVBackend(duplicity.backend.Backend):
 
     def sanitize_path(self, path):
         if path:
-            foldpath = re.compile('/+')
-            return foldpath.sub('/', path + '/')
+            foldpath = re.compile("/+")
+            return foldpath.sub("/", f"{path}/")
         else:
-            return '/'
+            return "/"
 
     def getText(self, nodelist):
         rc = ""
@@ -168,22 +177,18 @@ class WebDAVBackend(duplicity.backend.Backend):
         # reconnect on errors as a precaution, there are errors e.g.
         # "[Errno 32] Broken pipe" or SSl errors that render the connection unusable
         """
-        if not forced and self.conn \
-                and self.conn.host == self.parsed_url.hostname:
+        if not forced and self.conn and self.conn.host == self.parsed_url.hostname:
             return
 
-        log.Info(_("WebDAV create connection on '%s'") % (self.parsed_url.hostname))
+        log.Info(_("WebDAV create connection on '%s'") % self.parsed_url.hostname)
         self._close()
         # http schemes needed for redirect urls from servers
-        if self.parsed_url.scheme in ['webdav', 'http']:
-            self.conn = httplib.HTTPConnection(self.parsed_url.hostname, self.parsed_url.port)
-        elif self.parsed_url.scheme in ['webdavs', 'https']:
-            if globals.ssl_no_check_certificate:
-                self.conn = httplib.HTTPSConnection(self.parsed_url.hostname, self.parsed_url.port)
-            else:
-                self.conn = VerifiedHTTPSConnection(self.parsed_url.hostname, self.parsed_url.port)
+        if self.parsed_url.scheme in ["webdav", "http"]:
+            self.conn = http.client.HTTPConnection(self.parsed_url.hostname, self.parsed_url.port)
+        elif self.parsed_url.scheme in ["webdavs", "https"]:
+            self.conn = CustomHTTPSConnection(self.parsed_url.hostname, self.parsed_url.port)
         else:
-            raise FatalBackendException(_("WebDAV Unknown URI scheme: %s") % (self.parsed_url.scheme))
+            raise FatalBackendException(_("WebDAV Unknown URI scheme: %s") % self.parsed_url.scheme)
 
     def _close(self):
         if self.conn:
@@ -197,10 +202,10 @@ class WebDAVBackend(duplicity.backend.Backend):
         self._close()  # or we get previous request's data or exception
         self.connect()
 
-        quoted_path = urllib.quote(path, "/:~")
+        quoted_path = urllib.parse.quote(path, "/:~")
 
         if self.digest_challenge is not None:
-            self.headers['Authorization'] = self.get_digest_authorization(path)
+            self.headers["Authorization"] = self.get_digest_authorization(path)
 
         log.Info(_("WebDAV %s %s request with headers: %s ") % (method, quoted_path, self.headers))
         log.Info(_("WebDAV data length: %s ") % len(str(data)))
@@ -208,11 +213,11 @@ class WebDAVBackend(duplicity.backend.Backend):
         response = self.conn.getresponse()
         log.Info(_("WebDAV response status %s with reason '%s'.") % (response.status, response.reason))
         # resolve redirects and reset url on listing requests (they usually come before everything else)
-        if response.status in [301, 302] and method == 'PROPFIND':
-            redirect_url = response.getheader('location', None)
+        if response.status in [301, 302] and method == "PROPFIND":
+            redirect_url = response.getheader("location", None)
             response.close()
             if redirect_url:
-                log.Notice(_("WebDAV redirect to: %s ") % urllib.unquote(redirect_url))
+                log.Notice(_("WebDAV redirect to: %s ") % urllib.parse.unquote(redirect_url))
                 if redirected > 10:
                     raise FatalBackendException(_("WebDAV redirected 10 times. Giving up."))
                 self.parsed_url = duplicity.backend.ParsedUrl(redirect_url)
@@ -223,7 +228,7 @@ class WebDAVBackend(duplicity.backend.Backend):
         elif response.status == 401:
             response.read()
             response.close()
-            self.headers['Authorization'] = self.get_authorization(response, quoted_path)
+            self.headers["Authorization"] = self.get_authorization(response, quoted_path)
             log.Info(_("WebDAV retry request with authentification headers."))
             log.Info(_("WebDAV %s %s request2 with headers: %s ") % (method, quoted_path, self.headers))
             log.Info(_("WebDAV data length: %s ") % len(str(data)))
@@ -238,43 +243,53 @@ class WebDAVBackend(duplicity.backend.Backend):
         Fetches the auth header based on the requested method (basic or digest)
         """
         try:
-            auth_hdr = response.getheader('www-authenticate', '')
-            token, challenge = auth_hdr.split(' ', 1)
+            auth_hdr = response.getheader("www-authenticate", "")
+            token, challenge = auth_hdr.split(" ", 1)
         except ValueError:
             return None
-        if token.split(',')[0].lower() == 'negotiate':
+        if token.split(",")[0].lower() == "negotiate":
             try:
                 return self.get_kerberos_authorization()
             except ImportError:
-                log.warn(_("python-kerberos needed to use kerberos \
-                          authorization, falling back to basic auth."))
+                log.Warn(
+                    _(
+                        "python-kerberos needed to use kerberos \
+                          authorization, falling back to basic auth."
+                    )
+                )
                 return self.get_basic_authorization()
             except Exception as e:
-                log.warn(_("Kerberos authorization failed: %s.\
-                          Falling back to basic auth.") % e)
+                log.Warn(
+                    _(
+                        "Kerberos authorization failed: %s.\
+                          Falling back to basic auth."
+                    )
+                    % e
+                )
                 return self.get_basic_authorization()
-        elif token.lower() == 'basic':
+        elif token.lower() == "basic":
             return self.get_basic_authorization()
         else:
             self.digest_challenge = self.parse_digest_challenge(challenge)
             return self.get_digest_authorization(path)
 
     def parse_digest_challenge(self, challenge_string):
-        return urllib2.parse_keqv_list(urllib2.parse_http_list(challenge_string))
+        return urllib.request.parse_keqv_list(urllib.request.parse_http_list(challenge_string))
 
     def get_kerberos_authorization(self):
-        import kerberos
-        _, ctx = kerberos.authGSSClientInit("HTTP@%s" % self.conn.host)
+        import kerberos  # pylint: disable=import-error
+
+        _, ctx = kerberos.authGSSClientInit(f"HTTP@{self.conn.host}")
         kerberos.authGSSClientStep(ctx, "")
         tgt = kerberos.authGSSClientResponse(ctx)
-        return 'Negotiate %s' % tgt
+        return f"Negotiate {tgt}"
 
     def get_basic_authorization(self):
         """
         Returns the basic auth header
         """
-        auth_string = '%s:%s' % (self.username, self.password)
-        return 'Basic %s' % base64.encodestring(auth_string).strip()
+        auth_string = f"{self.username}:{self.password}"
+        return f"Basic {base64.b64encode(auth_string.encode()).strip().decode()}"
 
     def get_digest_authorization(self, path):
         """
@@ -282,25 +297,25 @@ class WebDAVBackend(duplicity.backend.Backend):
         """
         u = self.parsed_url
         if self.digest_auth_handler is None:
-            pw_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
+            pw_manager = urllib.request.HTTPPasswordMgrWithDefaultRealm()
             pw_manager.add_password(None, self.conn.host, self.username, self.password)
-            self.digest_auth_handler = urllib2.HTTPDigestAuthHandler(pw_manager)
+            self.digest_auth_handler = urllib.request.HTTPDigestAuthHandler(pw_manager)
 
         # building a dummy request that gets never sent,
         # needed for call to auth_handler.get_authorization
-        scheme = u.scheme == 'webdavs' and 'https' or 'http'
-        hostname = u.port and "%s:%s" % (u.hostname, u.port) or u.hostname
-        dummy_url = "%s://%s%s" % (scheme, hostname, path)
+        scheme = u.scheme == "webdavs" and "https" or "http"
+        hostname = u.port and f"{u.hostname}:{u.port}" or u.hostname
+        dummy_url = f"{scheme}://{hostname}{path}"
         dummy_req = CustomMethodRequest(self.conn._method, dummy_url)
         auth_string = self.digest_auth_handler.get_authorization(dummy_req, self.digest_challenge)
-        return 'Digest %s' % auth_string
+        return f"Digest {auth_string}"
 
     def _list(self):
         response = None
         try:
-            self.headers['Depth'] = "1"
+            self.headers["Depth"] = "1"
             response = self.request("PROPFIND", self.directory, self.listbody)
-            del self.headers['Depth']
+            del self.headers["Depth"]
             # if the target collection does not exist, create it.
             if response.status == 404:
                 response.close()  # otherwise next request fails with ResponseNotReady
@@ -314,12 +329,12 @@ class WebDAVBackend(duplicity.backend.Backend):
                 status = response.status
                 reason = response.reason
                 response.close()
-                raise BackendException("Bad status code %s reason %s." % (status, reason))
+                raise BackendException(f"Bad status code {status} reason {reason}.")
 
-            log.Debug("%s" % (document,))
+            log.Debug(f"{document}")
             dom = xml.dom.minidom.parseString(document)
             result = []
-            for href in dom.getElementsByTagName('d:href') + dom.getElementsByTagName('D:href'):
+            for href in dom.getElementsByTagNameNS("*", "href"):
                 filename = self.taste_href(href)
                 if filename:
                     result.append(filename)
@@ -335,16 +350,16 @@ class WebDAVBackend(duplicity.backend.Backend):
         dirs = self.directory.split("/")
         # url causes directory to start with /, but it might be given
         # with or without trailing / (which is required)
-        if dirs[-1] == '':
+        if dirs[-1] == "":
             dirs = dirs[0:-1]
         for i in range(1, len(dirs)):
-            d = "/".join(dirs[0:i + 1]) + "/"
+            d = f"{'/'.join(dirs[0:i + 1])}/"
 
-            self.headers['Depth'] = "1"
+            self.headers["Depth"] = "1"
             response = self.request("PROPFIND", d)
-            del self.headers['Depth']
+            del self.headers["Depth"]
 
-            log.Info("Checking existence dir %s: %d" % (d, response.status))
+            log.Info(f"Checking existence dir {d}: {int(response.status)}")
 
             if response.status == 404:
                 log.Info(_("Creating missing directory %s") % d)
@@ -361,10 +376,9 @@ class WebDAVBackend(duplicity.backend.Backend):
         @return: A matching filename, or None if the href did not match.
         """
         raw_filename = self.getText(href.childNodes).strip()
-        parsed_url = urlparse.urlparse(urllib.unquote(raw_filename))
+        parsed_url = urllib.parse.urlparse(urllib.parse.unquote(raw_filename))
         filename = parsed_url.path
-        log.Debug(_("WebDAV path decoding and translation: "
-                  "%s -> %s") % (raw_filename, filename))
+        log.Debug(_("WebDAV path decoding and translation: " "%s -> %s") % (raw_filename, filename))
 
         # at least one WebDAV server returns files in the form
         # of full URL:s. this may or may not be
@@ -372,35 +386,28 @@ class WebDAVBackend(duplicity.backend.Backend):
         # feel we want to bail out if the hostname
         # does not match until someone has looked into
         # what the WebDAV protocol mandages.
-        if parsed_url.hostname is not None \
-           and not (parsed_url.hostname == self.parsed_url.hostname):
-            m = "Received filename was in the form of a "\
-                "full url, but the hostname (%s) did "\
-                "not match that of the webdav backend "\
-                "url (%s) - aborting as a conservative "\
-                "safety measure. If this happens to you, "\
-                "please report the problem"\
-                "" % (parsed_url.hostname,
-                      self.parsed_url.hostname)
+        if parsed_url.hostname is not None and not (parsed_url.hostname == self.parsed_url.hostname):
+            m = (
+                f"Received filename was in the form of a full url, but the hostname ({parsed_url.hostname}) "
+                f"did not match that of the webdav backend url ({self.parsed_url.hostname}) - "
+                f"aborting as a conservative safety measure. If this happens to you, please report the problem"
+            )
             raise BackendException(m)
 
         if filename.startswith(self.directory):
-            filename = filename.replace(self.directory, '', 1)
+            filename = filename.replace(self.directory, "", 1)
             return filename
         else:
             return None
 
     def _get(self, remote_filename, local_path):
-        url = self.directory + remote_filename
+        url = self.directory + os.fsdecode(remote_filename)
         response = None
         try:
             target_file = local_path.open("wb")
             response = self.request("GET", url)
             if response.status == 200:
-                # data=response.read()
-                target_file.write(response.read())
-                # import hashlib
-                # log.Info("WebDAV GOT %s bytes with md5=%s" % (len(data),hashlib.md5(data).hexdigest()) )
+                shutil.copyfileobj(response, target_file)
                 assert not target_file.close()
                 response.close()
             else:
@@ -415,7 +422,7 @@ class WebDAVBackend(duplicity.backend.Backend):
                 response.close()
 
     def _put(self, source_path, remote_filename):
-        url = self.directory + remote_filename
+        url = self.directory + os.fsdecode(remote_filename)
         response = None
         try:
             source_file = source_path.open("rb")
@@ -436,7 +443,7 @@ class WebDAVBackend(duplicity.backend.Backend):
                 response.close()
 
     def _delete(self, filename):
-        url = self.directory + filename
+        url = self.directory + os.fsdecode(filename)
         response = None
         try:
             response = self.request("DELETE", url)
@@ -454,8 +461,9 @@ class WebDAVBackend(duplicity.backend.Backend):
             if response:
                 response.close()
 
+
 duplicity.backend.register_backend("http", WebDAVBackend)
 duplicity.backend.register_backend("https", WebDAVBackend)
 duplicity.backend.register_backend("webdav", WebDAVBackend)
 duplicity.backend.register_backend("webdavs", WebDAVBackend)
-duplicity.backend.uses_netloc.extend(['http', 'https', 'webdav', 'webdavs'])
+duplicity.backend.uses_netloc.extend(["http", "https", "webdav", "webdavs"])
