@@ -440,15 +440,21 @@ OptionKwargs = dict(
         type=set_log_file,
         help="Logging filename to use",
     ),
+    # log_timestamp is directly applied in SetLogTimestampAction(), not saved in config
     log_timestamp=dict(
-        action="store_true",
+        dest="",
+        action=SetLogTimestampAction,
         help="Whether to include timestamp and level in log",
         default=dflt(False),
     ),
     max_blocksize=dict(
         metavar=_("number"),
-        type=int,
-        help="Maximum block size for large files in MB",
+        type=round512,
+        help=(
+            "The integer square root of file length is used as the librsync block size\n"
+            "up to this maximum block size value in bytes if supplied.\n"
+            "Block size is rounded up to the nearest 512 byte boundary."
+        ),
         default=dflt(config.max_blocksize),
     ),
     metadata_sync_mode=dict(
