@@ -28,7 +28,9 @@ the second, the ROPath iterator is put into tar block form.
 """
 
 import io
+import math
 
+from duplicity import cli_util
 from duplicity import progress
 from duplicity import statistics
 from duplicity import dup_tarfile
@@ -743,15 +745,16 @@ def write_block_iter(block_iter, out_obj):
 
 def get_block_size(file_len):
     """
-    Return a reasonable block size to use on files of length file_len
+    Return a reasonable block size to use on files of length file_len.
 
-    If the block size is too big, deltas will be bigger than is
-    necessary.  If the block size is too small, making deltas and
-    patching can take a really long time.
+    Use the integer square root of file length as the librsync block size.
+    Minimum being 512 byte with no maximum unless --max-blocksize is supplied.
+
+    Block size is rounded up to the nearest 512 byte boundary.
     """
-    if file_len < 1024000:
-        return 512  # set minimum of 512 bytes
+
+    block_size = cli_util.round512(math.isqrt(file_len))
+    if config.max_blocksize:
+        return min(block_size, config.max_blocksize)
     else:
-        # Split file into about 2000 pieces, rounding to 512
-        file_blocksize = int((file_len / (2000 * 512))) * 512
-        return min(file_blocksize, config.max_blocksize)
+        return block_size
