@@ -141,19 +141,10 @@ def get_passphrase(n, action, for_signing=False):
     ]:
         return ""
 
-    # for a full backup, we don't need a password if
+    # for a full, inc, verify, we don't need a password if
     # there is no sign_key and there are recipients
     elif (
-        action == "full"
-        and (config.gpg_profile.recipients or config.gpg_profile.hidden_recipients)
-        and (not config.gpg_profile.sign_key or (not config.restart and not for_signing))
-    ):
-        return ""
-
-    # for an inc backup, we don't need a password if
-    # there is no sign_key and there are recipients
-    elif (
-        action == "inc"
+        action in ("full", "inc", "verify")
         and (config.gpg_profile.recipients or config.gpg_profile.hidden_recipients)
         and (not config.gpg_profile.sign_key or (not config.restart and not for_signing))
     ):
@@ -375,9 +366,9 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
         transfer_success = False
         manifest_written = False
 
-    def collect_put_results(bytes_written: int, backend_pooler, command2vol_map: Dict[int, CommandMetaData]) -> int:
-        commands_left, results = backend_pooler.collect_results()
-        for track_id, result in results.items():
+    def collect_put_results(bytes_written: int, backend_pooler, command2vol_map: Dict[int, CommandMetaData]):
+        for result in backend_pooler.results_since_last_call():
+            track_id = result.track_id
             size = result.result
             bytes_written += size
             progress.report_transfer(size, size)
@@ -391,31 +382,30 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
                 f"Transfer of {command2vol_map[track_id].path_obj.get_filename()} with id {track_id} and size "
                 f"{size} took {result.get_runtime()}"
             )
-        return commands_left
 
     def write_manifest_in_sequence(mf, mf_file, command2vol_map: Dict[int, CommandMetaData]):
         """
         Ensure volume info is written only if the volume transfer was successful and in sequence
         without gap otherwise missing volumes won't be detected and data is missing.
         """
-        i = iter(command2vol_map.keys())
-        info = command2vol_map[next(i)]
-        while info.manifest_written and info.transfer_success:
-            # skip all entries already written
+        try:
+            i = iter(command2vol_map.keys())
             info = command2vol_map[next(i)]
-        while info.transfer_success:
-            # if next volumes are tranferred
-            if not info.manifest_written:
-                mf.add_volume_info(info.vol_info)
-            info.manifest_written = True
-            try:
+            while info.manifest_written and info.transfer_success:
+                # skip all entries already written
                 info = command2vol_map[next(i)]
-            except StopIteration:
-                break
-            if info.vol_num == 1:
-                mf_file.to_partial()
-            else:
-                mf_file.flush()
+            while info.transfer_success:
+                # if next volumes are transferred
+                if not info.manifest_written:
+                    mf.add_volume_info(info.vol_info)
+                info.manifest_written = True
+                info = command2vol_map[next(i)]
+                if info.vol_num == 1:
+                    mf_file.to_partial()
+                else:
+                    mf_file.flush()
+        except StopIteration:
+            pass  # no manifest to update
 
     if not config.restart:
         # normal backup start
@@ -497,10 +487,9 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
             try:
                 # use process pool the write volumes in the background.
                 progress.report_transfer(0, tdp.getsize())
-                track_id = backend_pooler.command(backend.put_validated.__name__, args=(tdp, dest_filename))
+                track_id = backend_pooler.command_throttled(backend.put_validated.__name__, args=(tdp, dest_filename))
                 command2vol_map[track_id] = CommandMetaData(vol_num, tdp, vi)
-                commands_processing = collect_put_results(bytes_written, backend_pooler, command2vol_map)
-                log.Debug(f"Still {commands_processing} in the background pool.")
+                collect_put_results(bytes_written, backend_pooler, command2vol_map)
                 write_manifest_in_sequence(mf, man_outfp, command2vol_map)
             except (Exception, SystemExit) as e:
                 # ensure pool processes terminate clean
@@ -530,24 +519,28 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
             # wait for background commands, collect some stats and shutdown clean.
             log.Debug("Collecting remaining results from backend pool.")
             while True and backend_pooler:
-                commands_left = collect_put_results(bytes_written, backend_pooler, command2vol_map)
+                collect_put_results(bytes_written, backend_pooler, command2vol_map)
                 write_manifest_in_sequence(mf, man_outfp, command2vol_map)
-                if commands_left == 0:
+                if backend_pooler.get_queue_length() == 0:
                     break
                 else:
-                    log.Debug(f"Still {commands_left} commands left, waiting ...")
+                    log.Debug(f"Still {backend_pooler.get_queue_length()} commands left, waiting ...")
                     time.sleep(2)
             # print some stats, that can be used to track efficiency of concurrent transfers.
             stats = backend_pooler.get_stats()
             log.Info(
-                f'Transferred {stats["count"]} volumes, a volume took avg: {stats["avg"]}, '
-                'max: {stats["max"]}, min: {stats["min"]}'
+                f'Transferred {stats["count"]} volumes, a volume took avg: {stats["time"]["avg"]}, '
+                + f'max: {stats["time"]["max"]}, min: {stats["time"]["min"]}'
             )
-            # add stats to jsonstat. Not the most elegant way, but it is working.
-            diffdir.stats.stat_attrs += ("VolumeTransferTimeAVG", "VolumeTransferTimeMAX", "VolumeTransferTimeMIN")
-            diffdir.stats.set_stat("VolumeTransferTimeAVG", stats["avg"])
-            diffdir.stats.set_stat("VolumeTransferTimeMAX", stats["max"])
-            diffdir.stats.set_stat("VolumeTransferTimeMIN", stats["min"])
+            # double check that all volumes are transferred
+            if not any([x.transfer_success for x in command2vol_map.values()]) and not config.skipped_inc:
+                failed_volume_numbers = [
+                    x.vol_info.volume_number for x in command2vol_map.values() if not x.transfer_success
+                ]
+                log.FatalError(f"Volumes with number {failed_volume_numbers} were not transferred successful.")
+            # Add some stats, accessible with `--jsonstats`. Not the most elegant way, but it is working.
+            diffdir.stats.stat_attrs += ("ConcurrentTransferStats",)
+            diffdir.stats.set_stat("ConcurrentTransferStats", stats)
         finally:
             # make sure pool get terminated gracefully in any case.
             backend_pooler.shutdown()
@@ -899,9 +892,11 @@ def restore_get_patched_rop_iter(col_stats):
         volumes = manifest.get_containing_volumes(index)
         for vol_num in volumes:
             try:
-                yield restore_get_enc_fileobj(
+                fobj = restore_get_enc_fileobj(
                     backup_set.backend, backup_set.volume_name_dict[vol_num], manifest.volume_info_dict[vol_num]
                 )
+                if fobj is not None:
+                    yield fobj
             except BadVolumeException as e:
                 yield e
 
@@ -957,14 +952,15 @@ def restore_get_enc_fileobj(backend, filename, volume_info):
             log.Error(error_msg, code=log.ErrorCode.mismatched_hash)
     else:
         if config.ignore_errors:
-            exc = duplicity.errors.BadVolumeException(f"Hash mismatch for: {os.fsdecode(filename)}")
+            exc = BadVolumeException(f"Hash mismatch for: {os.fsdecode(filename)}")
             log.Warn(
                 _("IGNORED_ERROR: Warning: ignoring error as requested: %s: %s")
                 % (exc.__class__.__name__, util.uexc(exc))
             )
+            # Do not try to actually read it as it is corrupted!
+            return None
         else:
             log.FatalError(error_msg, code=log.ErrorCode.mismatched_hash)
-
     fileobj = tdp.filtered_open_with_delete("rb")
     if parseresults.encrypted and config.gpg_profile.sign_key:
         restore_add_sig_check(fileobj)
@@ -1253,7 +1249,7 @@ def sync_archive(col_stats):
 
     def copy_raw(src_iter, filename):
         """
-        Copy data from src_iter to file at fn
+        Copy data from src_iter to filename
         """
         file = open(filename, "wb")
         while True:
@@ -1455,14 +1451,14 @@ def check_resources(action):
         # Calculate space we need for at least 2 volumes of full or inc
         # plus about 30% of one volume for the signature files.
         freespace = stats.f_frsize * stats.f_bavail
-        needspace = ((config.concurrency + 1) * config.volsize) + int(0.30 * config.volsize)
+        needspace = ((config.concurrency + 2) * config.volsize) + int(0.30 * config.volsize)
         if freespace < needspace:
             log.FatalError(
-                _("Temp space has %d available, backup needs approx %d.") % (freespace, needspace),
+                _(f"Temp space has {freespace:,} available, backup needs approx {needspace:,}."),
                 log.ErrorCode.not_enough_freespace,
             )
         else:
-            log.Info(_("Temp has %d available, backup will use approx %d.") % (freespace, needspace))
+            log.Info(_(f"Temp has {freespace:,} available, backup will use approx {needspace:,}."))
 
         # Some environments like Cygwin run with an artificially
         # low value for max open files.  Check for safe number.
@@ -1472,17 +1468,13 @@ def check_resources(action):
             log.FatalError(_("Unable to get max open files."), log.ErrorCode.get_ulimit_failed)
         maxopen = min([l for l in (soft, hard) if l > -1])
         if maxopen < 1024:
-            try:
-              resource.setrlimit(resource.RLIMIT_NOFILE, (1024, hard))
-            except resource.error:
-              log.FatalError(
-                  _(
-                      "Max open files of %s is too low, should be >= 1024.\n"
-                      "Use 'ulimit -n 1024' or higher to correct.\n"
-                  )
-                  % (maxopen,),
-                  log.ErrorCode.maxopen_too_low,
-              )
+            log.FatalError(
+                _(
+                    f"Max open files of {maxopen} is too low, should be >= 1024.\n"
+                    f"Use 'ulimit -n 1024' or higher to correct.\n"
+                ),
+                log.ErrorCode.maxopen_too_low,
+            )
 
 
 def log_startup_parms(verbosity=log.INFO):
